<?php
require_once 'model.php';

class Cost extends Model {
    public static function getAll($filters = []) {
        $sql = "SELECT c.*, 
                CASE 
                    WHEN cr.relation_type = 'vehicle' THEN CONCAT(v.brand, ' ', v.model)
                    WHEN cr.relation_type = 'equipment' THEN e.name
                    WHEN cr.relation_type = 'employee' THEN CONCAT(em.first_name, ' ', em.last_name)
                    ELSE NULL
                END as related_name,
                cr.relation_type,
                c.is_general
                FROM costs c 
                LEFT JOIN cost_relations cr ON c.id = cr.cost_id
                LEFT JOIN vehicles v ON cr.relation_type = 'vehicle' AND cr.related_id = v.id
                LEFT JOIN equipment e ON cr.relation_type = 'equipment' AND cr.related_id = e.id
                LEFT JOIN employees em ON cr.relation_type = 'employee' AND cr.related_id = em.id
                WHERE 1=1";
        
        $params = [];

        if (!empty($filters['dateFrom'])) {
            $sql .= " AND c.date >= :dateFrom";
            $params['dateFrom'] = $filters['dateFrom'];
        }

        if (!empty($filters['dateTo'])) {
            $sql .= " AND c.date <= :dateTo";
            $params['dateTo'] = $filters['dateTo'];
        }

        if (!empty($filters['company'])) {
            $sql .= " AND c.company_id = :company";
            $params['company'] = $filters['company'];
        }

        if (!empty($filters['costType'])) {
            $sql .= " AND c.cost_type_id = :costType";
            $params['costType'] = $filters['costType'];
        }

        if (!empty($filters['status'])) {
            $sql .= " AND c.status = :status";
            $params['status'] = $filters['status'];
        }

        if (!empty($filters['keyword'])) {
            $sql .= " AND (c.description LIKE :keyword OR co.name LIKE :keyword OR ct.name LIKE :keyword)";
            $params['keyword'] = '%' . $filters['keyword'] . '%';
        }

        $sql .= " ORDER BY c.id DESC";

        return self::fetchAll($sql, $params);
    }

    public static function getById($id) {
        $sql = "SELECT c.*, ct.name as cost_type_name, co.name as company_name,
                cr.relation_type, cr.related_id,
                u.name as created_by_name
                FROM costs c 
                LEFT JOIN cost_types ct ON c.cost_type_id = ct.id 
                LEFT JOIN companies co ON c.company_id = co.id 
                LEFT JOIN cost_relations cr ON c.id = cr.cost_id
                LEFT JOIN users u ON c.created_by = u.id
                WHERE c.id = :id";
        
        try {
            return self::fetch($sql, ['id' => $id]);
        } catch (Exception $e) {
            error_log("Error in Cost::getById: " . $e->getMessage());
            throw $e;
        }
    }

    public static function add($data) {
        if (!isset($_SESSION['user_id'])) {
            throw new Exception('Brak zalogowanego użytkownika');
        }

        self::validateData($data);

        self::beginTransaction();

        try {
            $sql = "INSERT INTO costs (company_id, cost_type_id, amount, date, description, is_general, status, created_by) 
                    VALUES (:company_id, :cost_type_id, :amount, :date, :description, :is_general, :status, :created_by)";
            
            $sqlData = [
                'company_id' => $data['company_id'] ?? null,
                'cost_type_id' => $data['cost_type_id'] ?? null,
                'amount' => $data['amount'] ?? 0,
                'date' => $data['date'] ?? date('Y-m-d'),
                'description' => $data['description'] ?? '',
                'is_general' => isset($data['is_general']) ? 1 : 0,
                'status' => $data['status'] ?? 'nieopłacone',
                'created_by' => $_SESSION['user_id']
            ];

            if ($sqlData['company_id'] === null) {
                throw new Exception('Firma jest wymagana');
            }

            if (!strtotime($sqlData['date'])) {
                throw new Exception('Nieprawidłowy format daty');
            }

            self::execute($sql, $sqlData);
            $costId = self::lastInsertId();

            if (!$sqlData['is_general'] && !empty($data['relation_type']) && !empty($data['related_id'])) {
                self::addCostRelation($costId, $data['related_id'], $data['relation_type']);
            }

            self::commit();
            return $costId;
        } catch (Exception $e) {
            self::rollBack();
            error_log("Error in Cost::add: " . $e->getMessage());
            throw $e;
        }
    }

    public static function update($id, $data) {
        self::beginTransaction();
        try {
            // Update main cost data
            $sqlData = [
                'company_id' => $data['company_id'],
                'cost_type_id' => $data['cost_type_id'],
                'amount' => $data['amount'],
                'date' => $data['date'],
                'description' => $data['description'],
                'is_general' => $data['is_general'],
                'status' => $data['status'],
                'id' => $id
            ];
            
            $sql = "UPDATE costs SET 
                    company_id = :company_id, 
                    cost_type_id = :cost_type_id, 
                    amount = :amount, 
                    date = :date, 
                    description = :description, 
                    is_general = :is_general, 
                    status = :status 
                    WHERE id = :id";
            
            self::execute($sql, $sqlData);
            
            // Handle relations
            if ($data['is_general']) {
                // Delete all relations if cost is general
                self::deleteCostRelation($id);
            } elseif (!empty($data['relation_type']) && $data['relation_type'] !== 'none' && !empty($data['related_id'])) {
                // Update or create relation
                self::updateCostRelation($id, $data['related_id'], $data['relation_type']);
            }
            
            self::commit();
            return true;
        } catch (Exception $e) {
            self::rollBack();
            throw $e;
        }
    }

    public static function delete($id) {
        self::beginTransaction();

        try {
            self::deleteCostRelation($id);
            $sql = "DELETE FROM costs WHERE id = :id";
            self::execute($sql, ['id' => $id]);
            self::commit();
            return true;
        } catch (Exception $e) {
            self::rollBack();
            error_log("Error in Cost::delete: " . $e->getMessage());
            throw $e;
        }
    }

    private static function addCostRelation($costId, $relatedId, $relationType) {
        $sql = "INSERT INTO cost_relations (cost_id, related_id, relation_type) 
                VALUES (:cost_id, :related_id, :relation_type)";
        self::execute($sql, [
            'cost_id' => $costId,
            'related_id' => $relatedId,
            'relation_type' => $relationType
        ]);
    }

    private static function updateCostRelation($costId, $relatedId, $relationType) {
        $sql = "INSERT INTO cost_relations (cost_id, related_id, relation_type) 
                VALUES (:cost_id, :related_id, :relation_type)
                ON DUPLICATE KEY UPDATE 
                related_id = VALUES(related_id), 
                relation_type = VALUES(relation_type)";
        self::execute($sql, [
            'cost_id' => $costId,
            'related_id' => $relatedId,
            'relation_type' => $relationType
        ]);
    }

    private static function deleteCostRelation($costId) {
        $sql = "DELETE FROM cost_relations WHERE cost_id = :cost_id";
        self::execute($sql, ['cost_id' => $costId]);
    }

    public static function getFiltered($filters) {
        $query = "SELECT 
            c.*,
            cr.relation_type,
            cr.related_id,
            CASE 
                WHEN c.is_general = 1 THEN 'Ogólny'
                WHEN cr.relation_type = 'vehicle' THEN CONCAT(v.brand, ' ', v.model, ' (', v.registration_number, ')')
                WHEN cr.relation_type = 'equipment' THEN CONCAT(e.name, ' (SN: ', e.serial_number, ')')
                WHEN cr.relation_type = 'employee' THEN CONCAT(em.first_name, ' ', em.last_name)
                ELSE 'Ogólny'
            END as related_name
            FROM costs c 
            LEFT JOIN cost_relations cr ON c.id = cr.cost_id
            LEFT JOIN vehicles v ON (cr.relation_type = 'vehicle' AND cr.related_id = v.id)
            LEFT JOIN equipment e ON (cr.relation_type = 'equipment' AND cr.related_id = e.id)
            LEFT JOIN employees em ON (cr.relation_type = 'employee' AND cr.related_id = em.id)
            WHERE 1=1";

        $params = [];

        // Filtrowanie po typie kosztu
        if (!empty($filters['costType'])) {
            $query .= " AND c.cost_type_id = :costType";
            $params['costType'] = $filters['costType'];
        }

        // Filtrowanie po zakresie dat
        if (!empty($filters['dateFrom'])) {
            $query .= " AND c.date >= :dateFrom";
            $params['dateFrom'] = $filters['dateFrom'];
        }
        if (!empty($filters['dateTo'])) {
            $query .= " AND c.date <= :dateTo";
            $params['dateTo'] = $filters['dateTo'];
        }

        // Filtrowanie po typie relacji
        if (!empty($filters['relationType'])) {
            if ($filters['relationType'] === 'general') {
                $query .= " AND c.is_general = 1";
            } else {
                $query .= " AND cr.relation_type = :relationType";
                $params['relationType'] = $filters['relationType'];
            }
        }

        // Filtrowanie po powiązanym obiekcie
        if (!empty($filters['relatedId'])) {
            $query .= " AND cr.related_id = :relatedId";
            $params['relatedId'] = $filters['relatedId'];
        }

        $query .= " ORDER BY c.date DESC, c.id DESC";

        error_log("Generated SQL query: " . $query);
        error_log("Query parameters: " . print_r($params, true));

        $results = self::fetchAll($query, $params);
        error_log("Query results: " . print_r($results, true));

        return $results;
    }

    private static function validateData($data) {
        $errors = [];
        
        // Sprawdzanie wymaganych pól
        if (empty($data['company_id'])) {
            $errors[] = 'Firma jest wymagana';
        }
        
        if (!isset($data['amount'])) {
            $errors[] = 'Kwota jest wymagana';
        } elseif ($data['amount'] <= 0) {
            $errors[] = 'Kwota musi być większa od 0';
        }
        
        // Sprawdzanie formatu daty
        if (!empty($data['date'])) {
            if (!strtotime($data['date'])) {
                $errors[] = 'Nieprawidłowy format daty';
            }
        }
        
        if (!empty($errors)) {
            throw new Exception(implode(', ', $errors));
        }
        
        return true;
    }

    public static function calculateTotal($filters = []) {
        $sql = "SELECT SUM(amount) as total FROM costs c 
                LEFT JOIN cost_relations cr ON c.id = cr.cost_id
                WHERE 1=1";
        
        $params = [];

        if (!empty($filters['dateFrom'])) {
            $sql .= " AND c.date >= :dateFrom";
            $params['dateFrom'] = $filters['dateFrom'];
        }

        if (!empty($filters['dateTo'])) {
            $sql .= " AND c.date <= :dateTo";
            $params['dateTo'] = $filters['dateTo'];
        }

        if (!empty($filters['company'])) {
            $sql .= " AND c.company_id = :company";
            $params['company'] = $filters['company'];
        }

        if (!empty($filters['costType'])) {
            $sql .= " AND c.cost_type_id = :costType";
            $params['costType'] = $filters['costType'];
        }

        if (!empty($filters['relationType'])) {
            if ($filters['relationType'] === 'general') {
                $sql .= " AND c.is_general = 1";
            } else {
                $sql .= " AND cr.relation_type = :relationType";
                $params['relationType'] = $filters['relationType'];
            }
        }

        if (!empty($filters['relatedId'])) {
            $sql .= " AND cr.related_id = :relatedId";
            $params['relatedId'] = $filters['relatedId'];
        }

        $result = self::fetch($sql, $params);
        return $result['total'] ?? 0;
    }
}
