<?php
// app/models/document.php
require_once 'model.php';

class Document extends Model
{
    public static function getAll($filters = [])
    {
        error_log("Rozpoczęto wykonywanie funkcji getAll w klasie Document");
        $sql = "SELECT d.id, d.title, d.created_at, d.related_entity_type, d.related_entity_id,
                       dc.name as category_name, c.name as company_name,
                       CASE 
                         WHEN d.related_entity_type = 'employee' THEN CONCAT(e.first_name, ' ', e.last_name)
                         WHEN d.related_entity_type = 'vehicle' THEN v.registration_number
                         WHEN d.related_entity_type = 'equipment' THEN eq.name
                         ELSE NULL
                       END as related_entity_name,
                                      d.is_generated
                FROM documents d
                LEFT JOIN document_categories dc ON d.category_id = dc.id
                LEFT JOIN companies c ON d.company_id = c.id
                LEFT JOIN employees e ON d.related_entity_type = 'employee' AND d.related_entity_id = e.id
                LEFT JOIN vehicles v ON d.related_entity_type = 'vehicle' AND d.related_entity_id = v.id
                LEFT JOIN equipment eq ON d.related_entity_type = 'equipment' AND d.related_entity_id = eq.id";

        $where = [];
        $params = [];
        if (isset($filters['is_generated'])) {
            $where[] = "d.is_generated = :is_generated";
            $params['is_generated'] = $filters['is_generated'];
        }
        if (!empty($filters['category_id'])) {
            $where[] = "d.category_id = :category_id";
            $params['category_id'] = $filters['category_id'];
        }
        if (!empty($filters['company_id'])) {
            $where[] = "d.company_id = :company_id";
            $params['company_id'] = $filters['company_id'];
        }
        if (!empty($filters['related_entity_type']) && $filters['related_entity_type'] !== 'all') {
            $where[] = "d.related_entity_type = :related_entity_type";
            $params['related_entity_type'] = $filters['related_entity_type'];
        }
        if (!empty($filters['search'])) {
            $where[] = "(d.title LIKE :search OR d.description LIKE :search)";
            $params['search'] = '%' . $filters['search'] . '%';
        }

        if (!empty($where)) {
            $sql .= " WHERE " . implode(" AND ", $where);
        }

        $sql .= " ORDER BY d.created_at DESC";

        error_log("SQL Query: " . $sql);
        error_log("Params: " . print_r($params, true));

        $result = self::fetchAll($sql, $params);
        error_log("Result count: " . count($result));
        error_log("Zakończono wykonywanie funkcji getAll w klasie Document");
        return $result;
    }

    public static function getById($id)
    {
        error_log("Rozpoczęto wykonywanie funkcji getById w klasie Document");
        $sql = "SELECT d.*, dc.name as category_name, c.name as company_name, u.name as created_by_name,
                       CASE 
                         WHEN d.related_entity_type = 'employee' THEN CONCAT(e.first_name, ' ', e.last_name)
                         WHEN d.related_entity_type = 'vehicle' THEN v.registration_number
                         ELSE NULL
                       END as related_entity_name
                FROM documents d
                LEFT JOIN document_categories dc ON d.category_id = dc.id
                LEFT JOIN companies c ON d.company_id = c.id
                LEFT JOIN users u ON d.created_by = u.id
                LEFT JOIN employees e ON d.related_entity_type = 'employee' AND d.related_entity_id = e.id
                LEFT JOIN vehicles v ON d.related_entity_type = 'vehicle' AND d.related_entity_id = v.id
                WHERE d.id = :id";
        $result = self::fetch($sql, ['id' => $id]);
        error_log("Zakończono wykonywanie funkcji getById w klasie Document");
        return $result;
    }

    public static function add($data)
    {
        error_log("Rozpoczęto wykonywanie funkcji add w klasie Document");
        error_log("Otrzymane dane: " . print_r($data, true));
        $defaultValues = [
            'description' => null,
            'company_id' => null,
            'policy_number' => null,
            'policy_start_date' => null,
            'policy_end_date' => null,
            'insurance_company' => null,
            'is_generated' => false  // Domyślnie false dla ręcznie dodawanych dokumentów
        ];
        $data = array_merge($defaultValues, $data);
        // Przygotowanie dat
        $policyStartDate = !empty($data['policy_start_date']) ? $data['policy_start_date'] : null;
        $policyEndDate = !empty($data['policy_end_date']) ? $data['policy_end_date'] : null;

        $sql = "INSERT INTO documents (title, description, file_path, file_type, category_id, company_id, 
                related_entity_type, related_entity_id, created_by, created_at, updated_at, 
                policy_number, policy_start_date, policy_end_date, insurance_company, is_generated, template_id) 
                VALUES (:title, :description, :file_path, :file_type, :category_id, :company_id,
                :related_entity_type, :related_entity_id, :created_by, NOW(), NOW(),
                :policy_number, :policy_start_date, :policy_end_date, :insurance_company, :is_generated, :template_id)";

        $params = [
            'title' => $data['title'],
            'description' => $data['description'],
            'file_path' => $data['file_path'],
            'file_type' => $data['file_type'],
            'category_id' => $data['category_id'],
            'company_id' => $data['company_id'],
            'related_entity_type' => $data['related_entity_type'],
            'related_entity_id' => $data['related_entity_type'] === 'none' ? null : ($data['related_entity_id'] ?? null),
            'created_by' => $data['created_by'],
            'policy_number' => $data['policy_number'] ?? null,
            'policy_start_date' => $policyStartDate,
            'policy_end_date' => $policyEndDate,
            'insurance_company' => $data['insurance_company'] ?? null,
            'is_generated' => $data['is_generated'] ?? false,
            'template_id' => $data['template_id'] ?? null
        ];

        error_log("Przygotowane parametry: " . print_r($params, true));

        try {
            $result = self::execute($sql, $params);
            if ($result) {
                return self::lastInsertId();
            }
            return false;
        } catch (PDOException $e) {
            error_log("Error in Document::add: " . $e->getMessage());
            return false;
        }
    }
    public static function update($id, $data)
    {
        error_log("Rozpoczęto wykonywanie funkcji update w klasie Document");
        error_log("ID dokumentu: " . $id);
        error_log("Dane do aktualizacji: " . print_r($data, true));

        $allowedFields = ['title', 'description', 'category_id', 'company_id', 'related_entity_type', 'related_entity_id', 'file_path', 'file_type', 'policy_number', 'policy_start_date', 'policy_end_date', 'insurance_company'];
        $updateFields = [];
        $updateData = [];

        foreach ($allowedFields as $field) {
            if (isset($data[$field])) {
                if ($field === 'policy_start_date' || $field === 'policy_end_date') {
                    $updateFields[] = "$field = " . (!empty($data[$field]) ? ":$field" : "NULL");
                    if (!empty($data[$field])) {
                        $updateData[$field] = $data[$field];
                    }
                } else {
                    $updateFields[] = "$field = :$field";
                    $updateData[$field] = $data[$field];
                }
            }
        }

        // Obsługa nowego pliku
        if (isset($data['new_file_path']) && isset($data['new_file_type'])) {
            $updateFields[] = "file_path = :new_file_path";
            $updateFields[] = "file_type = :new_file_type";
            $updateData['new_file_path'] = $data['new_file_path'];
            $updateData['new_file_type'] = $data['new_file_type'];
        }

        if (empty($updateFields)) {
            error_log("Brak pól do aktualizacji");
            return false;
        }

        $sql = "UPDATE documents SET " . implode(', ', $updateFields) . ", updated_at = NOW() WHERE id = :id";

        $updateData['id'] = $id;

        error_log("SQL Query: " . $sql);
        error_log("Parametry zapytania: " . print_r($updateData, true));

        try {
            $result = self::execute($sql, $updateData);
            error_log("Rezultat zapytania: " . ($result ? "sukces" : "niepowodzenie"));
            return $result;
        } catch (PDOException $e) {
            error_log("Błąd PDO: " . $e->getMessage());
            throw $e;
        }
    }

    public static function delete($id)
    {
        error_log("Rozpoczęto wykonywanie funkcji delete w klasie Document");
        $document = self::getById($id);
        if ($document && file_exists($document['file_path'])) {
            unlink($document['file_path']);
        }

        $sql = "DELETE FROM documents WHERE id = :id";
        $result = self::execute($sql, ['id' => $id]);
        error_log("Zakończono wykonywanie funkcji delete w klasie Document");
        return $result;
    }

    public static function getForEmployee($employeeId)
    {
        error_log("Rozpoczęto wykonywanie funkcji getForEmployee w klasie Document");
        $sql = "SELECT d.*, dc.name as category_name
            FROM documents d
            LEFT JOIN document_categories dc ON d.category_id = dc.id
            WHERE d.related_entity_type = 'employee' AND d.related_entity_id = :employee_id
            ORDER BY d.created_at DESC";
        $result = self::fetchAll($sql, ['employee_id' => $employeeId]);
        error_log("Zakończono wykonywanie funkcji getForEmployee w klasie Document");
        return $result;
    }

    public static function getForVehicle($vehicleId)
    {
        error_log("Rozpoczęto wykonywanie funkcji getForVehicle w klasie Document");
        $sql = "SELECT d.*, dc.name as category_name
            FROM documents d
            LEFT JOIN document_categories dc ON d.category_id = dc.id
            WHERE d.related_entity_type = 'vehicle' AND d.related_entity_id = :vehicle_id
            ORDER BY d.created_at DESC";
        $result = self::fetchAll($sql, ['vehicle_id' => $vehicleId]);
        error_log("Zakończono wykonywanie funkcji getForVehicle w klasie Document");
        return $result;
    }

    public static function addPolicy($data)
    {
        error_log("Rozpoczęto wykonywanie funkcji addPolicy w klasie Document");
        try {
            $sql = "INSERT INTO documents (title, description, file_path, file_type, category_id, company_id,
                    related_entity_type, related_entity_id, created_by, policy_number, policy_start_date, 
                    policy_end_date, insurance_company)
                    VALUES (:title, :description, :file_path, :file_type, :category_id, :company_id,
                    :related_entity_type, :related_entity_id, :created_by, :policy_number, :policy_start_date, 
                    :policy_end_date, :insurance_company)";

            error_log("SQL Query: " . $sql);
            error_log("Data before filtering: " . print_r($data, true));

            $expectedFields = [
                'title',
                'description',
                'file_path',
                'file_type',
                'category_id',
                'company_id',
                'related_entity_type',
                'related_entity_id',
                'created_by',
                'policy_number',
                'policy_start_date',
                'policy_end_date',
                'insurance_company'
            ];

            $filteredData = array_intersect_key($data, array_flip($expectedFields));

            foreach ($expectedFields as $field) {
                if (!isset($filteredData[$field])) {
                    $filteredData[$field] = null;
                }
            }

            error_log("Filtered Data: " . print_r($filteredData, true));

            $result = self::execute($sql, $filteredData);
            error_log('Rezultat dodawania polisy: ' . print_r($result, true));
            error_log("Zakończono wykonywanie funkcji addPolicy w klasie Document");
            return $result;
        } catch (PDOException $e) {
            error_log('PDOException w addPolicy: ' . $e->getMessage());
            throw $e;
        }
    }

    public static function updatePolicy($id, $data)
    {
        error_log("Rozpoczęto wykonywanie funkcji updatePolicy w klasie Document");
        try {
            $allowedFields = ['title', 'description', 'category_id', 'policy_number', 'policy_start_date', 'policy_end_date', 'insurance_company'];
            $updateFields = [];
            $updateData = [];

            foreach ($allowedFields as $field) {
                if (isset($data[$field])) {
                    $updateFields[] = "$field = :$field";
                    $updateData[$field] = $data[$field];
                }
            }

            if (empty($updateFields)) {
                throw new Exception("No fields to update");
            }

            $sql = "UPDATE documents SET " . implode(', ', $updateFields) . " WHERE id = :id";

            $updateData['id'] = $id;

            error_log("Updating policy with SQL: " . $sql);
            error_log("Updating policy with data: " . print_r($updateData, true));

            $result = self::execute($sql, $updateData);

            error_log("Update result: " . ($result ? "success" : "failure"));
            error_log("Zakończono wykonywanie funkcji updatePolicy w klasie Document");
            return $result;
        } catch (Exception $e) {
            error_log('Exception in updatePolicy: ' . $e->getMessage());
            throw $e;
        }
    }

    public static function getPoliciesForVehicle($vehicleId)
    {
        error_log("Rozpoczęto wykonywanie funkcji getPoliciesForVehicle w klasie Document");
        $sql = "SELECT d.*, dc.name as category_name
                FROM documents d
                LEFT JOIN document_categories dc ON d.category_id = dc.id
                WHERE d.related_entity_type = 'vehicle' 
                AND d.related_entity_id = :vehicle_id
                AND dc.name IN ('Polisa OC', 'Polisa AC')
                ORDER BY d.policy_end_date DESC";
        $result = self::fetchAll($sql, ['vehicle_id' => $vehicleId]);
        error_log("Zakończono wykonywanie funkcji getPoliciesForVehicle w klasie Document");
        return $result;
    }
}