<?php

require_once 'model.php';

class Task extends Model {
    public static function getAll($filters = []) {
        $sql = "SELECT t.*, u.name as user_name 
                FROM tasks t
                LEFT JOIN users u ON t.assigned_to = u.id
                WHERE 1=1";
        $params = [];
        
        if (!empty($filters['search'])) {
            $sql .= " AND (t.title LIKE :search OR t.description LIKE :search)";
            $params['search'] = "%{$filters['search']}%";
        }
        
        if (!empty($filters['status'])) {
            if ($filters['status'] === 'not_done') {
                $sql .= " AND t.status != :done_status";
                $params['done_status'] = self::STATUS_DONE;
            } else {
                $sql .= " AND t.status = :status";
                $params['status'] = $filters['status'];
            }
        }
        
        // Dodajemy nowy filtr dla przypisanych zadań
        if (!empty($filters['assignment'])) {
            switch ($filters['assignment']) {
                case 'my':
                    $sql .= " AND t.assigned_to = :user_id";
                    $params['user_id'] = $_SESSION['user_id'];
                    break;
                case 'my_and_general':
                    $sql .= " AND (t.assigned_to = :user_id OR t.assigned_to IS NULL)";
                    $params['user_id'] = $_SESSION['user_id'];
                    break;
                // case 'all' - nie potrzebuje dodatkowego warunku
            }
        }
        
        $sql .= " ORDER BY t.due_date ASC";
        
        return self::fetchAll($sql, $params);
    }

    public static function getById($id) {
        $sql = "SELECT t.*, u.name as user_name 
                FROM tasks t
                LEFT JOIN users u ON t.assigned_to = u.id 
                WHERE t.id = :id";
        return self::fetch($sql, ['id' => $id]);
    }

    public static function add($title, $description, $priority, $due_date, $status, $created_by, $assigned_to = null) {
        // Walidacja danych przed dodaniem
        if (empty($title)) {
            throw new Exception('Tytuł jest wymagany');
        }

        if (!in_array($priority, array_keys(self::getPriorities()))) {
            throw new Exception('Nieprawidłowy priorytet');
        }

        if (!empty($due_date)) {
            if (!strtotime($due_date)) {
                throw new Exception('Nieprawidłowy format daty');
            }
            $due_date = date('Y-m-d', strtotime($due_date));
        } else {
            $due_date = null;
        }

        if (!in_array($status, array_keys(self::getStatuses()))) {
            throw new Exception('Nieprawidłowy status');
        }

        if (empty($created_by)) {
            throw new Exception('Pole created_by jest wymagane');
        }

        $sql = "INSERT INTO tasks (title, description, priority, due_date, status, created_by, assigned_to, created_at, updated_at) 
                VALUES (:title, :description, :priority, :due_date, :status, :created_by, :assigned_to, NOW(), NOW())";
                
        return self::execute($sql, [
            'title' => $title,
            'description' => $description,
            'priority' => $priority,
            'due_date' => $due_date,
            'status' => $status,
            'created_by' => $created_by,
            'assigned_to' => $assigned_to
        ]);
    }

    public static function update($id, $title, $description, $priority, $due_date, $status, $assigned_to = null) {
        // Dodajemy tę samą walidację co w metodzie add
        $due_date = trim($due_date) === '' ? NULL : $due_date;
        
        $sql = "UPDATE tasks SET 
                title = :title,
                description = :description,
                priority = :priority,
                due_date = :due_date,
                status = :status,
                assigned_to = :assigned_to,
                updated_at = CURRENT_TIMESTAMP
                WHERE id = :id";
                
        return self::execute($sql, [
            'id' => $id,
            'title' => $title,
            'description' => $description,
            'priority' => $priority,
            'due_date' => $due_date,
            'status' => $status,
            'assigned_to' => $assigned_to
        ]);
    }

    public static function delete($id) {
        // Zamiast usuwać, aktualizujemy status na 'done'
        $sql = "UPDATE tasks SET 
                status = :status,
                updated_at = NOW() 
                WHERE id = :id";
        return self::execute($sql, [
            'id' => $id,
            'status' => self::STATUS_DONE
        ]);
    }

    // Dodaj stałe dla statusów
    const STATUS_TODO = 'todo';
    const STATUS_IN_PROGRESS = 'in_progress';
    const STATUS_DONE = 'done';
    const DEFAULT_STATUS_FILTER = 'not_done';
    const DEFAULT_ASSIGNMENT_FILTER = 'my_and_general';
    
    const PRIORITY_LOW = 'low';
    const PRIORITY_MEDIUM = 'medium';
    const PRIORITY_HIGH = 'high';
    
    public static function getStatuses() {
        $statuses = [
            self::STATUS_TODO => 'Do zrobienia',
            self::STATUS_IN_PROGRESS => 'W trakcie',
            self::STATUS_DONE => 'Zakończone'
        ];
        error_log('Statuses: ' . print_r($statuses, true));
        return $statuses;
    }

    public static function getPriorities() {
        return [
            self::PRIORITY_LOW => 'Niski',
            self::PRIORITY_MEDIUM => 'Średni',
            self::PRIORITY_HIGH => 'Wysoki'
        ];
    }

    public static function getRecentTasks($limit = 5) {
        $sql = "SELECT t.* 
                FROM tasks t
                WHERE t.status != :status
                ORDER BY 
                    CASE 
                        WHEN t.due_date < CURRENT_DATE THEN 1
                        WHEN t.due_date = CURRENT_DATE THEN 2
                        ELSE 3
                    END,
                    t.priority DESC,
                    t.due_date ASC
                LIMIT :limit";
        
        return self::fetchAll($sql, [
            'status' => self::STATUS_DONE,
            'limit' => $limit
        ]);
    }
}
