<?php

require_once 'model.php';

class Vehicle extends Model
{
    public static function getAllWithDetails($searchTerm = '')
    {
        $sql = "SELECT v.*, c.name as company_name,
                       vmm.mileage as last_mileage,
                       vit.expiration_date as next_technical_inspection,
                       vis.expiration_date as next_service_inspection
                FROM vehicles v
                LEFT JOIN companies c ON v.company_id = c.id
                LEFT JOIN (
                    SELECT vehicle_id, MAX(mileage_date) as max_date
                    FROM vehicle_monthly_mileages
                    GROUP BY vehicle_id
                ) vmm_max ON v.id = vmm_max.vehicle_id
                LEFT JOIN vehicle_monthly_mileages vmm ON vmm.vehicle_id = v.id AND vmm.mileage_date = vmm_max.max_date
                LEFT JOIN (
                    SELECT vehicle_id, MAX(inspection_date) as max_date
                    FROM vehicle_inspections
                    WHERE inspection_type = 'techniczny'
                    GROUP BY vehicle_id
                ) vi_tech ON v.id = vi_tech.vehicle_id
                LEFT JOIN vehicle_inspections vit ON vit.vehicle_id = v.id AND vit.inspection_date = vi_tech.max_date AND vit.inspection_type = 'techniczny'
                LEFT JOIN (
                    SELECT vehicle_id, MAX(inspection_date) as max_date
                    FROM vehicle_inspections
                    WHERE inspection_type = 'serwisowy'
                    GROUP BY vehicle_id
                ) vi_serv ON v.id = vi_serv.vehicle_id
                LEFT JOIN vehicle_inspections vis ON vis.vehicle_id = v.id AND vis.inspection_date = vi_serv.max_date AND vis.inspection_type = 'serwisowy'
                WHERE v.brand LIKE :searchTerm OR v.model LIKE :searchTerm OR v.registration_number LIKE :searchTerm
                ORDER BY v.brand, v.model";
        return self::fetchAll($sql, ['searchTerm' => '%' . $searchTerm . '%']);
    }

    public static function getAll()
    {
        $sql = "SELECT v.*, c.name as company_name
            FROM vehicles v
            LEFT JOIN companies c ON v.company_id = c.id
            ORDER BY v.brand, v.model";
        return self::fetchAll($sql);
    }

    public static function getById($id)
    {
        $sql = "SELECT v.*, c.name as company_name FROM vehicles v 
                LEFT JOIN companies c ON v.company_id = c.id 
                WHERE v.id = :id";
        return self::fetch($sql, ['id' => $id]);
    }



    public static function add($brand, $model, $year, $registration_number, $vin, $purchase_date, $company_id, $created_by, $first_registration_date)
    {
        $sql = "INSERT INTO vehicles (brand, model, year, registration_number, vin, purchase_date, company_id, created_by, first_registration_date, created_at, updated_at) 
            VALUES (:brand, :model, :year, :registration_number, :vin, :purchase_date, :company_id, :created_by, :first_registration_date, NOW(), NOW())";
        return self::execute($sql, [
            'brand' => $brand,
            'model' => $model,
            'year' => $year,
            'registration_number' => $registration_number ?: null,
            'vin' => $vin,
            'purchase_date' => $purchase_date,
            'company_id' => $company_id,
            'created_by' => $created_by,
            'first_registration_date' => $first_registration_date ?: null
        ]);
    }

    public static function update($id, $data)
    {
        $sql = "UPDATE vehicles SET 
            brand = :brand, 
            model = :model, 
            year = :year, 
            registration_number = :registration_number, 
            vin = :vin, 
            purchase_date = :purchase_date, 
            company_id = :company_id,
            first_registration_date = :first_registration_date,
            updated_at = NOW() 
            WHERE id = :id";

        $params = [
            'id' => $id,
            'brand' => $data['brand'],
            'model' => $data['model'],
            'year' => $data['year'],
            'registration_number' => $data['registration_number'] ?: null,
            'vin' => $data['vin'],
            'purchase_date' => $data['purchase_date'],
            'company_id' => $data['company_id'],
            'first_registration_date' => $data['first_registration_date'] ?: null
        ];

        return self::execute($sql, $params);
    }
    public static function getMileageById($id)
    {
        $sql = "SELECT * FROM vehicle_monthly_mileages WHERE id = :id";
        return self::fetch($sql, ['id' => $id]);
    }

    public static function getMileagesForVehicle($vehicleId)
    {
        $sql = "SELECT * FROM vehicle_monthly_mileages 
            WHERE vehicle_id = :vehicle_id 
            ORDER BY mileage_date DESC";
        return self::fetchAll($sql, ['vehicle_id' => $vehicleId]);
    }

    public static function getLatestMileage($id)
    {
        $sql = "SELECT mileage, mileage_date FROM vehicle_monthly_mileages 
                    WHERE vehicle_id = :id 
                    ORDER BY mileage_date DESC 
                    LIMIT 1";
        return self::fetch($sql, ['id' => $id]);
    }

    public static function updateMileage($id, $mileage, $mileage_month)
    {
        // Konwertuj datę na ostatni dzień wybranego miesiąca
        $lastDayOfMonth = date('Y-m-t', strtotime($mileage_month . '-01'));

        // Sprawdź, czy istnieje już wpis na ten miesiąc
        $existingEntry = self::fetch(
            "SELECT * FROM vehicle_monthly_mileages 
                                          WHERE vehicle_id = :id AND mileage_date = :mileage_date",
            ['id' => $id, 'mileage_date' => $lastDayOfMonth]
        );

        if ($existingEntry) {
            // Jeśli wpis istnieje, zwróć informację o istniejącym wpisie
            return [
                'success' => false,
                'existing' => true,
                'existingMileage' => $existingEntry['mileage'],
                'newMileage' => $mileage,
                'mileageDate' => $lastDayOfMonth
            ];
        }

        // Pobierz poprzedni przebieg
        $previousMileage = self::fetch(
            "SELECT mileage FROM vehicle_monthly_mileages 
                                            WHERE vehicle_id = :id AND mileage_date < :mileage_date 
                                            ORDER BY mileage_date DESC LIMIT 1",
            ['id' => $id, 'mileage_date' => $lastDayOfMonth]
        );

        $mileageDifference = $previousMileage ? ($mileage - $previousMileage['mileage']) : $mileage;

        if ($mileageDifference < 0) {
            return ['success' => false, 'error' => 'Nowy przebieg nie może być mniejszy od poprzedniego.'];
        }

        $sql = "INSERT INTO vehicle_monthly_mileages (vehicle_id, mileage_date, mileage, mileage_difference, created_by, created_at, updated_at)
                    VALUES (:id, :mileage_date, :mileage, :mileage_difference, :created_by, NOW(), NOW())";

        $params = [
            'id' => $id,
            'mileage_date' => $lastDayOfMonth,
            'mileage' => $mileage,
            'mileage_difference' => $mileageDifference,
            'created_by' => $_SESSION['user_id']
        ];

        $result = self::execute($sql, $params);

        if ($result) {
            if ($mileageDifference > 5000) {
                return ['success' => true, 'warning' => 'Różnica przebiegu przekracza 5000 km.'];
            }
            return ['success' => true];
        }

        return ['success' => false, 'error' => 'Nie udało się zaktualizować przebiegu.'];
    }
    public static function confirmUpdateMileage($id, $mileage, $mileage_date)
    {
        $sql = "UPDATE vehicle_monthly_mileages 
                    SET mileage = :mileage,
                        mileage_difference = :mileage - COALESCE((
                            SELECT mileage 
                            FROM (SELECT mileage FROM vehicle_monthly_mileages 
                                  WHERE vehicle_id = :id AND mileage_date < :mileage_date 
                                  ORDER BY mileage_date DESC LIMIT 1) AS prev
                        ), 0),
                        updated_at = NOW()
                    WHERE vehicle_id = :id AND mileage_date = :mileage_date";

        $params = [
            'id' => $id,
            'mileage_date' => $mileage_date,
            'mileage' => $mileage
        ];

        $result = self::execute($sql, $params);

        if ($result) {
            return ['success' => true];
        }

        return ['success' => false, 'error' => 'Nie udało się zaktualizować przebiegu.'];
    }
    public static function getNearestMileage($vehicleId, $date)
    {
        $sql = "SELECT mileage, mileage_date 
                    FROM vehicle_monthly_mileages 
                    WHERE vehicle_id = :vehicle_id 
                    ORDER BY ABS(DATEDIFF(mileage_date, :date)) 
                    LIMIT 1";
        return self::fetch($sql, ['vehicle_id' => $vehicleId, 'date' => $date]);
    }
    public static function editMileage($id, $mileage, $mileage_date, $vehicle_id)
    {
        $sql = "UPDATE vehicle_monthly_mileages 
                SET mileage = :mileage,
                    mileage_difference = :mileage - COALESCE((
                        SELECT mileage 
                        FROM (SELECT mileage FROM vehicle_monthly_mileages 
                              WHERE vehicle_id = :vehicle_id AND mileage_date < :mileage_date 
                              ORDER BY mileage_date DESC LIMIT 1) AS prev
                    ), 0),
                    updated_at = NOW()
                WHERE id = :id";
    
        $params = [
            'id' => $id,
            'mileage' => $mileage,
            'mileage_date' => $mileage_date,
            'vehicle_id' => $vehicle_id
        ];
    
        $result = self::execute($sql, $params);
    
        if ($result) {
            return ['success' => true];
        }
    
        return ['success' => false, 'error' => 'Nie udało się zaktualizować przebiegu.'];
    }
    public static function getMileageVehicleId($id)
    {
        $sql = "SELECT vehicle_id FROM vehicle_monthly_mileages WHERE id = :id";
        $result = self::fetch($sql, ['id' => $id]);
        return $result ? $result['vehicle_id'] : null;
    }
    public static function deleteMileage($id)
    {
        $sql = "DELETE FROM vehicle_monthly_mileages WHERE id = :id";
        $result = self::execute($sql, ['id' => $id]);

        if ($result) {
            return ['success' => true];
        }

        return ['success' => false, 'error' => 'Nie udało się usunąć przebiegu.'];
    }
    public static function delete($id)
    {
        try {
            self::beginTransaction();
            error_log("Rozpoczęcie usuwania pojazdu ID: " . $id);

            // 1. Usuwanie przeglądów pojazdu
            $sql = "DELETE FROM vehicle_inspections WHERE vehicle_id = :id";
            self::execute($sql, ['id' => $id]);
            error_log("Usunięto powiązane przeglądy");

            // 2. Usuwanie przebiegów miesięcznych
            $sql = "DELETE FROM vehicle_monthly_mileages WHERE vehicle_id = :id";
            self::execute($sql, ['id' => $id]);
            error_log("Usunięto powiązane przebiegi");

            // 3. Usuwanie kosztów operacyjnych
            $sql = "DELETE FROM vehicle_operating_costs WHERE vehicle_id = :id";
            self::execute($sql, ['id' => $id]);
            error_log("Usunięto powiązane koszty operacyjne");

            // 4. Usuwanie napraw
            $sql = "DELETE FROM vehicle_repairs WHERE vehicle_id = :id";
            self::execute($sql, ['id' => $id]);
            error_log("Usunięto powiązane naprawy");

            // 5. Aktualizacja dokumentów (zamiast usuwania)
            $sql = "UPDATE documents SET related_entity_id = NULL WHERE related_entity_type = 'vehicle' AND related_entity_id = :id";
            self::execute($sql, ['id' => $id]);
            error_log("Zaktualizowano powiązane dokumenty");

            // 6. Usuwanie powiązań z kosztami
            $sql = "DELETE FROM cost_relations WHERE relation_type = 'vehicle' AND related_id = :id";
            self::execute($sql, ['id' => $id]);
            error_log("Usunięto powiązania z kosztami");

            // 7. Na końcu usuwamy sam pojazd
            $sql = "DELETE FROM vehicles WHERE id = :id";
            $result = self::execute($sql, ['id' => $id]);

            if ($result) {
                error_log("Pojazd został pomyślnie usunięty");
                self::commit();
                return [
                    'success' => true,
                    'message' => 'Pojazd został pomyślnie usunięty wraz ze wszystkimi powiązanymi danymi'
                ];
            } else {
                error_log("Błąd podczas usuwania pojazdu");
                self::rollBack();
                return [
                    'success' => false,
                    'message' => 'Błąd podczas usuwania pojazdu'
                ];
            }
        } catch (PDOException $e) {
            error_log("Błąd PDO podczas usuwania pojazdu: " . $e->getMessage());
            self::rollBack();
            return [
                'success' => false,
                'message' => "Wystąpił błąd podczas usuwania pojazdu: " . $e->getMessage()
            ];
        }
    }
}
