# System logowania w projekcie KOBAS (panel.kobas.pl)

## Przegląd

System logowania w projekcie KOBAS został zaprojektowany, aby ułatwić debugowanie i monitorowanie aplikacji. System wykorzystuje kombinację natywnych funkcji PHP do logowania oraz niestandardowych funkcji, które zapewniają bardziej szczegółowe i ustrukturyzowane logi.

## Konfiguracja logowania

Konfiguracja logowania jest zdefiniowana w pliku `/var/www/html/panel/public/index.php`:

```php
// Konfiguracja logowania dla środowiska deweloperskiego
ini_set('display_errors', 0); // Nie pokazuj błędów użytkownikom
ini_set('display_startup_errors', 0);
// Loguj wszystkie błędy oprócz powiadomień i przestarzałych funkcji
error_reporting(E_ALL & ~E_NOTICE & ~E_DEPRECATED);
ini_set('log_errors', 1);
ini_set('error_log', '/var/www/html/panel/error.log');
```

Główne ustawienia:
- Błędy nie są wyświetlane użytkownikom (`display_errors = 0`)
- Logowane są wszystkie błędy oprócz powiadomień (E_NOTICE) i przestarzałych funkcji (E_DEPRECATED)
- Logi są zapisywane do pliku `/var/www/html/panel/error.log`

## Funkcje logowania

### 1. Standardowy handler błędów

```php
function custom_error_handler($errno, $errstr, $errfile, $errline) {
    error_log("Error [$errno] $errstr on line $errline in file $errfile");
}
set_error_handler("custom_error_handler");
```

Ten handler jest używany do przechwytywania i logowania standardowych błędów PHP.

### 2. Funkcja debug_log

```php
function debug_log($message, $data = null) {
    $log = "[" . date('Y-m-d H:i:s') . "] " . $message;
    if ($data !== null) {
        $log .= ": " . print_r($data, true);
    }
    error_log($log);
}
```

Funkcja `debug_log()` jest głównym narzędziem do logowania w projekcie. Zapewnia:
- Automatyczne dodawanie znacznika czasu
- Możliwość logowania zarówno wiadomości tekstowych, jak i struktur danych
- Czytelny format logów

## Logowanie w modelach

Bazowy model (`/var/www/html/panel/app/models/model.php`) wykorzystuje funkcję `debug_log()` do logowania operacji bazodanowych:

### Logowanie połączenia z bazą danych

```php
try {
    self::$db = new PDO(
        "mysql:host={$config['host']};dbname={$config['dbname']};charset=utf8mb4",
        $config['user'],
        $config['password'],
        [PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION]
    );
} catch (PDOException $e) {
    debug_log("Nie można połączyć się z bazą danych", $e->getMessage());
    throw new Exception("Problem z połączeniem do bazy danych.");
}
```

### Logowanie zapytań SQL

```php
protected static function execute($sql, $params = []) {
    $db = self::getDB();
    $stmt = $db->prepare($sql);
    
    // Logowanie tylko dla zapytań innych niż SELECT
    if (!preg_match('/^SELECT/i', $sql)) {
        debug_log("Model::execute - SQL", $sql);
        debug_log("Model::execute - Params", $params);
    }
    
    $stmt->execute($params);
    return $stmt;
}
```

Zapytania SELECT nie są logowane w metodzie `execute()`, aby zmniejszyć ilość logów dla często wykonywanych zapytań odczytu.

### Logowanie wyników zapytań

```php
protected static function fetchAll($sql, $params = []) {
    try {
        debug_log("Model::execute - SQL", $sql);
        debug_log("Model::execute - Params", $params);
        
        $stmt = self::execute($sql, $params);
        $result = $stmt->fetchAll(PDO::FETCH_ASSOC);
        
        debug_log("Model::fetchAll - Liczba zwróconych wyników", count($result));
        
        return $result;
    } catch (PDOException $e) {
        debug_log("SQL Error", [
            'message' => $e->getMessage(),
            'query' => $sql,
            'params' => $params
        ]);
        throw $e;
    }
}
```

## Jak korzystać z systemu logowania

### Logowanie prostej wiadomości

```php
debug_log("Rozpoczęcie przetwarzania formularza");
```

### Logowanie z danymi

```php
debug_log("Dane formularza", $_POST);
```

### Logowanie błędu

```php
try {
    // kod, który może generować błędy
} catch (Exception $e) {
    debug_log("Wystąpił błąd", $e->getMessage());
}
```

## Plik logów

Logi są zapisywane w pliku `/var/www/html/panel/error.log`. Plik ten powinien być regularnie monitorowany i rotowany, aby zapobiec jego nadmiernemu rozrostowi.

### Przykładowy format logów

```
[2025-03-03 20:45:12] Model::execute - SQL: SELECT * FROM users WHERE id = ?
[2025-03-03 20:45:12] Model::execute - Params: Array ( [0] => 1 )
[2025-03-03 20:45:12] Model::fetchAll - Liczba zwróconych wyników: 1
```

## Zalecenia dotyczące logowania

1. **Używaj funkcji debug_log zamiast error_log** - zapewnia to spójny format i więcej informacji.
2. **Loguj istotne operacje** - logowanie powinno obejmować istotne operacje, takie jak logowanie użytkowników, modyfikacje danych, błędy itp.
3. **Unikaj logowania danych wrażliwych** - hasła, tokeny i inne dane wrażliwe nie powinny być logowane.
4. **Monitoruj rozmiar pliku logów** - duży plik logów może wpływać na wydajność serwera.

## Rozwiązywanie problemów

Jeśli logi nie są generowane lub są niepełne, sprawdź:
1. Uprawnienia do pliku `/var/www/html/panel/error.log` - Apache musi mieć uprawnienia do zapisu.
2. Konfigurację PHP w pliku `/etc/php/7.4/apache2/php.ini` - upewnij się, że `log_errors = On`.
3. Czy funkcja `debug_log()` jest dostępna w kontekście, w którym jest wywoływana. 