class TaskManager {
    constructor() {
        this.searchInput = document.getElementById('search-input');
        this.taskList = document.getElementById('task-list');
        this.statusSelect = document.getElementById('status-select');
        this.assignmentSelect = document.getElementById('assignment-select');
        this.clearSearchIcon = document.getElementById('clear-search-icon');
        this.debounceTimer = null;

        this.initializeEventListeners();
    }

    initializeEventListeners() {
        if (this.searchInput) {
            this.searchInput.addEventListener('input', () => this.handleSearch());
            this.searchInput.addEventListener('keypress', (e) => {
                if (e.key === 'Enter') {
                    e.preventDefault();
                    this.filterTasks();
                }
            });
        }

        if (this.statusSelect) {
            this.statusSelect.addEventListener('change', () => this.filterTasks());
        }

        if (this.assignmentSelect) {
            this.assignmentSelect.addEventListener('change', () => this.filterTasks());
        }

        if (this.clearSearchIcon) {
            this.clearSearchIcon.addEventListener('click', () => this.clearSearch());
        }

        this.toggleClearIcon();
        this.filterTasks();
    }

    handleSearch() {
        this.toggleClearIcon();
        clearTimeout(this.debounceTimer);
        this.debounceTimer = setTimeout(() => {
            this.filterTasks();
        }, 300);
    }

    toggleClearIcon() {
        if (this.clearSearchIcon) {
            this.clearSearchIcon.style.display = this.searchInput.value ? 'block' : 'none';
        }
    }

    clearSearch() {
        if (this.searchInput) {
            this.searchInput.value = '';
            this.toggleClearIcon();
            this.filterTasks();
        }
    }

    filterTasks() {
        const searchTerm = this.searchInput ? this.searchInput.value.trim() : '';
        const selectedStatus = this.statusSelect ? this.statusSelect.value : '';
        const selectedAssignment = this.assignmentSelect ? this.assignmentSelect.value : '';

        let url = '/tasks';
        const params = new URLSearchParams();
        
        if (searchTerm) {
            params.append('search', searchTerm);
        }
        if (selectedStatus) {
            params.append('status', selectedStatus);
        }
        if (selectedAssignment && selectedAssignment !== 'all') {
            params.append('assignment', selectedAssignment);
        }
        
        if (params.toString()) {
            url += '?' + params.toString();
        }

        fetch(url)
            .then(response => response.text())
            .then(html => {
                const parser = new DOMParser();
                const doc = parser.parseFromString(html, 'text/html');
                const newTaskList = doc.getElementById('task-list');
                if (newTaskList && this.taskList) {
                    this.taskList.innerHTML = newTaskList.innerHTML;
                } else if (this.taskList) {
                    this.taskList.innerHTML = '<tr><td colspan="7">Brak wyników</td></tr>';
                }
            })
            .catch(error => {
                console.error('Error:', error);
                if (this.taskList) {
                    this.taskList.innerHTML = '<tr><td colspan="7">Wystąpił błąd podczas wczytywania danych</td></tr>';
                }
            });
    }
}

// Inicjalizacja po załadowaniu dokumentu
document.addEventListener('DOMContentLoaded', () => {
    new TaskManager();
});
