<?php
session_start();
define('ROOT_PATH', dirname(__DIR__));
require_once ROOT_PATH . '/app/models/model.php';
require_once ROOT_PATH . '/app/models/user.php';
require_once ROOT_PATH . '/app/models/task.php';
require_once ROOT_PATH . '/config/database.php';

class TaskTest {
    private $testResults = [];

    public function runTests() {
        try {
            // Symulacja zalogowanego użytkownika
            $_SESSION['user_id'] = 1;
            
            echo "Rozpoczynam testy modułu zadań...\n\n";

            // Test funkcjonalności Task
            $this->testGetAll();
            $this->testFiltering();
            $this->testValidation();
            $this->testAddTask();
            $this->testEditTask();
            $this->testDeleteTask();

            $this->displayResults();

        } catch (Exception $e) {
            echo "Błąd podczas testów: " . $e->getMessage() . "\n";
        }
    }

    private function testGetAll() {
        try {
            $tasks = Task::getAll();
            $this->addResult('Pobieranie wszystkich zadań',
                is_array($tasks),
                'Pobrano listę zadań',
                'Błąd podczas pobierania zadań'
            );
        } catch (Exception $e) {
            $this->addResult('Pobieranie wszystkich zadań', false, '', $e->getMessage());
        }
    }

    private function testFiltering() {
        try {
            $filters = [
                'search' => 'test', // Przykładowe wyszukiwanie
                'status' => 'todo'  // Przykładowy status
            ];
            
            $tasks = Task::getAll($filters);
            $this->addResult('Filtrowanie zadań',
                is_array($tasks),
                'Filtrowanie działa poprawnie',
                'Błąd podczas filtrowania'
            );
        } catch (Exception $e) {
            $this->addResult('Filtrowanie zadań', false, '', $e->getMessage());
        }
    }

    private function testValidation() {
        try {
            $testCases = [
                [
                    'data' => [
                        'title' => '',
                        'description' => 'Opis zadania',
                        'priority' => 'medium',
                        'due_date' => '2023-12-31',
                        'status' => 'todo',
                        'assigned_to' => null,
                        'created_by' => $_SESSION['user_id'] ?? 1
                    ],
                    'expectedError' => 'Tytuł jest wymagany'
                ],
                [
                    'data' => [
                        'title' => 'Testowe zadanie',
                        'description' => 'Opis zadania',
                        'priority' => 'medium',
                        'due_date' => 'invalid-date',
                        'status' => 'todo',
                        'assigned_to' => null,
                        'created_by' => $_SESSION['user_id'] ?? 1
                    ],
                    'expectedError' => 'Nieprawidłowy format daty'
                ]
            ];

            $allTestsPassed = true;
            $errors = [];

            foreach ($testCases as $index => $testCase) {
                try {
                    Task::add(
                        $testCase['data']['title'],
                        $testCase['data']['description'],
                        $testCase['data']['priority'],
                        $testCase['data']['due_date'],
                        $testCase['data']['status'],
                        $testCase['data']['assigned_to'],
                        $testCase['data']['created_by']
                    );
                    $allTestsPassed = false;
                    $errors[] = "Test case " . ($index + 1) . " powinien zakończyć się błędem";
                } catch (Exception $e) {
                    if (strpos($e->getMessage(), $testCase['expectedError']) === false) {
                        $allTestsPassed = false;
                        $errors[] = "Test case " . ($index + 1) . " zakończony nieoczekiwanym błędem: " . $e->getMessage();
                    }
                }
            }

            $this->addResult('Walidacja danych',
                $allTestsPassed,
                'Wszystkie przypadki testowe przeszły poprawnie',
                'Błędy walidacji: ' . implode(', ', $errors)
            );
        } catch (Exception $e) {
            $this->addResult('Walidacja danych', false, '', $e->getMessage());
        }
    }

    private function testAddTask() {
        try {
            $taskData = [
                'title' => 'Nowe zadanie',
                'description' => 'Opis nowego zadania',
                'priority' => 'medium',
                'due_date' => '2023-12-31',
                'status' => 'todo',
                'assigned_to' => null,
                'created_by' => $_SESSION['user_id'] ?? 1
            ];
            $result = Task::add(...array_values($taskData));
            $this->addResult('Dodawanie zadania',
                $result,
                'Zadanie dodane pomyślnie',
                'Błąd podczas dodawania zadania'
            );
        } catch (Exception $e) {
            $this->addResult('Dodawanie zadania', false, '', $e->getMessage());
        }
    }

    private function testEditTask() {
        try {
            // Zakładamy, że zadanie o ID 1 istnieje
            $taskData = [
                'title' => 'Zaktualizowane zadanie',
                'description' => 'Opis zaktualizowanego zadania',
                'priority' => 'high',
                'due_date' => '2023-12-31',
                'status' => 'in_progress',
                'assigned_to' => null,
                'created_by' => $_SESSION['user_id'] ?? 1
            ];
            $result = Task::update(1, ...array_values($taskData)); // Zaktualizuj zadanie o ID 1
            $this->addResult('Edycja zadania',
                $result,
                'Zadanie zaktualizowane pomyślnie',
                'Błąd podczas edycji zadania'
            );
        } catch (Exception $e) {
            $this->addResult('Edycja zadania', false, '', $e->getMessage());
        }
    }

    private function testDeleteTask() {
        try {
            // Zakładamy, że zadanie o ID 1 istnieje
            $result = Task::delete(1); // Usuń zadanie o ID 1
            $this->addResult('Usuwanie zadania',
                $result,
                'Zadanie usunięte pomyślnie',
                'Błąd podczas usuwania zadania'
            );
        } catch (Exception $e) {
            $this->addResult('Usuwanie zadania', false, '', $e->getMessage());
        }
    }

    private function addResult($testName, $success, $successMessage, $errorMessage) {
        $this->testResults[] = [
            'name' => $testName,
            'success' => $success,
            'message' => $success ? $successMessage : $errorMessage
        ];
    }

    private function displayResults() {
        echo "\nWyniki testów:\n";
        echo "=============\n\n";

        foreach ($this->testResults as $result) {
            echo $result['name'] . ": " . 
                 ($result['success'] ? "✅ SUKCES" : "❌ BŁĄD") . 
                 "\nKomentarz: " . $result['message'] . "\n\n";
        }
    }
}

// Uruchomienie testów
$tester = new TaskTest();
$tester->runTests(); 