<?php
session_start();
define('ROOT_PATH', dirname(__DIR__));
require_once ROOT_PATH . '/app/models/model.php';
require_once ROOT_PATH . '/app/models/Cost.php';
require_once ROOT_PATH . '/config/database.php';

class CostFiltersTest {
    private $testResults = [];
    private $testCostId;

    public function runTests() {
        try {
            echo "Rozpoczynam testy filtrów modułu kosztów...\n\n";

            // Symulujemy sesję
            $_SESSION['user_id'] = 1;

            // Testy różnych scenariuszy filtrowania
            $this->testFilterByCostType();
            $this->testFilterByDateRange();
            $this->testFilterByRelationType();
            $this->testFilterByGeneralCosts();
            $this->testFilterCombination();
            $this->testEmptyFilters();

            $this->displayResults();

        } catch (Exception $e) {
            echo "Błąd podczas testów: " . $e->getMessage() . "\n";
        }
    }

    private function testFilterByCostType() {
        try {
            $filters = ['costType' => 16]; // Przykładowy typ kosztu
            $costs = Cost::getFiltered($filters);
            
            $allCostsMatchType = true;
            foreach ($costs as $cost) {
                if ($cost['cost_type_id'] != 16) {
                    $allCostsMatchType = false;
                    break;
                }
            }

            $this->addResult(
                'Filtrowanie po typie kosztu',
                $allCostsMatchType,
                'Wszystkie koszty mają prawidłowy typ',
                'Znaleziono koszty o nieprawidłowym typie'
            );
        } catch (Exception $e) {
            $this->addResult('Filtrowanie po typie kosztu', false, '', $e->getMessage());
        }
    }

    private function testFilterByDateRange() {
        try {
            $filters = [
                'dateFrom' => '2024-01-01',
                'dateTo' => '2024-12-31'
            ];
            
            $costs = Cost::getFiltered($filters);
            
            $allCostsInRange = true;
            foreach ($costs as $cost) {
                $costDate = strtotime($cost['date']);
                if ($costDate < strtotime($filters['dateFrom']) || 
                    $costDate > strtotime($filters['dateTo'])) {
                    $allCostsInRange = false;
                    break;
                }
            }

            $this->addResult(
                'Filtrowanie po zakresie dat',
                $allCostsInRange,
                'Wszystkie koszty są w zadanym zakresie dat',
                'Znaleziono koszty poza zakresem dat'
            );
        } catch (Exception $e) {
            $this->addResult('Filtrowanie po zakresie dat', false, '', $e->getMessage());
        }
    }

    private function testFilterByRelationType() {
        try {
            $filters = ['relationType' => 'vehicle'];
            $costs = Cost::getFiltered($filters);
            
            $allCostsHaveVehicle = true;
            foreach ($costs as $cost) {
                if ($cost['relation_type'] !== 'vehicle' || empty($cost['related_name'])) {
                    $allCostsHaveVehicle = false;
                    break;
                }
            }

            $this->addResult(
                'Filtrowanie po typie relacji (pojazd)',
                $allCostsHaveVehicle,
                'Wszystkie koszty są powiązane z pojazdami',
                'Znaleziono koszty niepowiązane z pojazdami'
            );
        } catch (Exception $e) {
            $this->addResult('Filtrowanie po typie relacji', false, '', $e->getMessage());
        }
    }

    private function testFilterByGeneralCosts() {
        try {
            $filters = ['isGeneral' => 1];
            $costs = Cost::getFiltered($filters);
            
            $allCostsAreGeneral = true;
            foreach ($costs as $cost) {
                if ($cost['is_general'] != 1) {
                    $allCostsAreGeneral = false;
                    break;
                }
            }

            $this->addResult(
                'Filtrowanie kosztów ogólnych',
                $allCostsAreGeneral,
                'Wszystkie koszty są ogólne',
                'Znaleziono koszty niebędące ogólnymi'
            );
        } catch (Exception $e) {
            $this->addResult('Filtrowanie kosztów ogólnych', false, '', $e->getMessage());
        }
    }

    private function testFilterCombination() {
        try {
            $filters = [
                'costType' => 16,
                'dateFrom' => '2024-01-01',
                'dateTo' => '2024-12-31',
                'isGeneral' => 0,
                'relationType' => 'vehicle'
            ];
            
            $costs = Cost::getFiltered($filters);
            
            $allFiltersMatch = true;
            foreach ($costs as $cost) {
                if ($cost['cost_type_id'] != 16 ||
                    strtotime($cost['date']) < strtotime($filters['dateFrom']) ||
                    strtotime($cost['date']) > strtotime($filters['dateTo']) ||
                    $cost['is_general'] != 0 ||
                    $cost['relation_type'] !== 'vehicle') {
                    $allFiltersMatch = false;
                    break;
                }
            }

            $this->addResult(
                'Filtrowanie - kombinacja filtrów',
                $allFiltersMatch,
                'Wszystkie koszty spełniają wszystkie kryteria',
                'Znaleziono koszty niespełniające wszystkich kryteriów'
            );
        } catch (Exception $e) {
            $this->addResult('Filtrowanie - kombinacja filtrów', false, '', $e->getMessage());
        }
    }

    private function testEmptyFilters() {
        try {
            $costs = Cost::getFiltered([]);
            $this->addResult(
                'Filtrowanie bez filtrów',
                !empty($costs),
                'Pobrano wszystkie koszty',
                'Błąd podczas pobierania wszystkich kosztów'
            );
        } catch (Exception $e) {
            $this->addResult('Filtrowanie bez filtrów', false, '', $e->getMessage());
        }
    }

    private function addResult($testName, $success, $successMessage, $errorMessage) {
        $this->testResults[] = [
            'name' => $testName,
            'success' => $success,
            'message' => $success ? $successMessage : $errorMessage
        ];
    }

    private function displayResults() {
        echo "\nWyniki testów filtrów:\n";
        echo "===================\n\n";

        foreach ($this->testResults as $result) {
            echo $result['name'] . ": " . 
                 ($result['success'] ? "✅ SUKCES" : "❌ BŁĄD") . 
                 "\nKomentarz: " . $result['message'] . "\n\n";
        }
    }
}

// Uruchomienie testów
$tester = new CostFiltersTest();
$tester->runTests(); 